@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* dbus-server-protected.h Used by subclasses of DBusServer object (internal to D-Bus implementation)
  *
  * Copyright (C) 2002  Red Hat Inc.
@@ -27,6 +27,7 @@
 #include <dbus/dbus-internals.h>
 #include <dbus/dbus-threads-internal.h>
 #include <dbus/dbus-server.h>
+#include <dbus/dbus-address.h>
 #include <dbus/dbus-timeout.h>
 #include <dbus/dbus-watch.h>
 #include <dbus/dbus-resources.h>
@@ -36,16 +37,6 @@
 DBUS_BEGIN_DECLS
 
 typedef struct DBusServerVTable DBusServerVTable;
-typedef union DBusGUID DBusGUID;
-
-/**
- * A server's globally unique ID
- */
-union DBusGUID
-{
-  dbus_uint32_t as_uint32s[4];
-  unsigned char as_bytes[16];
-};
 
 /**
  * Virtual table to be implemented by all server "subclasses"
@@ -121,6 +112,18 @@ void        _dbus_server_toggle_timeout (DBusServer             *server,
 void        _dbus_server_ref_unlocked   (DBusServer             *server);
 void        _dbus_server_unref_unlocked (DBusServer             *server);
 
+typedef enum
+{
+  DBUS_SERVER_LISTEN_NOT_HANDLED, /**< we aren't in charge of this address type */
+  DBUS_SERVER_LISTEN_OK,          /**< we set up the listen */
+  DBUS_SERVER_LISTEN_BAD_ADDRESS, /**< malformed address */
+  DBUS_SERVER_LISTEN_DID_NOT_CONNECT /**< well-formed address but failed to set it up */
+} DBusServerListenResult;
+
+DBusServerListenResult _dbus_server_listen_platform_specific (DBusAddressEntry  *entry,
+                                                              DBusServer       **server_p,
+                                                              DBusError         *error);
+
 #ifdef DBUS_DISABLE_CHECKS
 #define TOOK_LOCK_CHECK(server)
 #define RELEASING_LOCK_CHECK(server)
