@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* dbus-server-unix.c Server implementation for Unix network protocols.
  *
  * Copyright (C) 2002, 2003, 2004  Red Hat Inc.
@@ -43,7 +43,7 @@
  * Sets error if the result is not OK.
  * 
  * @param entry an address entry
- * @param a new DBusServer, or #NULL on failure.
+ * @param server_p location to store a new DBusServer, or #NULL on failure.
  * @param error location to store rationale for failure on bad address
  * @returns the outcome
  * 
@@ -209,7 +209,7 @@ _dbus_server_new_for_domain_socket (const char     *path,
       goto failed_1;
     }
   
-  server = _dbus_server_new_for_socket (listen_fd, &address);
+  server = _dbus_server_new_for_socket (&listen_fd, 1, &address);
   if (server == NULL)
     {
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
